<?php
/*
 *
 * ElbiaAdmin PHP-lib version 3.1.9
 * Copyright (c) 2009-2024, Jozef Sudolsky, ELBIA, s. r. o. (jozef@sudolsky.sk)
 * All rights reserved.
 * BSD Licence
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     1. Redistributions of source code must retain the above copyright notice,
 *        this list of conditions and the following disclaimer.
 *
 *     2. Redistributions in binary form must reproduce the above copyright
 *        notice, this list of conditions and the following disclaimer in the
 *        documentation and/or other materials provided with the distribution.
 *
 *     3. Neither the name of ElbiaAdmin PHP-lib nor the names of its
 *        contributors may be used to endorse or promote products derived from
 *        this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 */

class ElbiaAdmin {

	const EXPIRED_STATE = "expired";
	const BLOCKED_STATE = "blocked";
	const LOGIN_OK_STATE = "ok";
	const CHPASS_OK_STATE = "ok";
	const TRAN_OK_STATE = "ok";
	const DIRECTOR_WSDL = "ws.elbiaadmin.sk/?wsdl";

	private $relogin_on_expire = null;
	protected $token = null;
	private $clients = null;
	protected $login = null;
	protected $password = null;
	private $web_backend = null;
	private $mail_backend = null;
	private $dns_backend = null;
	private $nic_backend = null;
	private $sms_backend = null;
	private $invoicing_backend = null;

	function __construct($login=null, $password=null, $relogin=true, $token=null) {
		$this->login = $login;
		$this->password = $password;
		$this->relogin_on_expire = $relogin;
		$this->token = $token;
		$this->clients = array();
	}

	private function _init_client($wsdl) {
		$client = new SoapClient("https://" . $wsdl);
		//$client = new SoapClient("https://" . $wsdl, array("keep_alive" => false));
		$wsdl_types = $client->__getTypes();
		$types = array();
		for ($i = 0; $i < count($wsdl_types); $i += 1) {
			$type = explode("{\n ", $wsdl_types[$i]);
			if (count($type) > 1) {
				$name = explode(" ", $type[0]);
				$types[$name[1]] = array();
				$params = explode("\n ", $type[1]);
				for ($j = 0; $j < count($params); $j += 1) {
					$param = explode(" ", $params[$j]);
					$types[$name[1]][] = str_replace(";", "", str_replace("\n}", "", $param[1]));
				}
			}
		}
		$wsdl_funcs = $client->__getFunctions();
		$funcs = array();
		for ($i = 0; $i < count($wsdl_funcs); $i += 1) {
			$func = explode(" ", $wsdl_funcs[$i], 2);
			$func = explode("(", $func[1], 2);
			$params = explode(" ", $func[1], 2);
			$params = str_replace("$", "", str_replace(")", "", $params[1]));
			if (array_key_exists($params, $types)) {
				$funcs[$func[0]] = $types[$params];
			} else {
				$funcs[$func[0]] = array();
			}
		}
		return array("client" => $client, "functions" => $funcs);
	}

	private function _get_client($type) {
		if (! array_key_exists($type, $this->clients)) {
			if ($type == "DI") {
				$wsdl = self::DIRECTOR_WSDL;
			} elseif ($type == "W") {
				$wsdl = $this->web_backend;
			} elseif ($type == "M") {
				$wsdl = $this->mail_backend;
			} elseif ($type == "D") {
				$wsdl = $this->dns_backend;
			} elseif ($type == "N") {
				$wsdl = $this->nic_backend;
			} elseif ($type == "S") {
				$wsdl = $this->sms_backend;
			} elseif ($type == "I") {
				$wsdl = $this->invoicing_backend;
			}
			$this->clients[$type] = $this->_init_client($wsdl);
		}
		return $this->clients[$type];
	}

	private function _do_soap($type, $command, $params) {
		$client = $this->_get_client($type);
		// sluzba invoicing
		if ($type == "I") {
			$result = $client["client"]->__soapCall($command, $params);
			return (object) $result;
		} else {
			$params2 = array();
			for ($i = 0; $i < count($client["functions"][$command]); $i += 1) {
				if (array_key_exists($i, $params)) {
					$params2[$client["functions"][$command][$i]] = $params[$i];
				}
			}
			try {
				$result = $client["client"]->__soapCall($command, array($params2));
			} catch (Exception $e) {
				if ($e->faultstring == "DTD are not supported by SOAP" || $e->faultstring == "Service Temporarily Unavailable") {
					sleep(5);
					$result = $client["client"]->__soapCall($command, array($params2));
				} else {
					throw $e;
				}
			}
			$prop = array_keys(get_object_vars($result));
			return $result->{$prop[0]};
		}
	}

	function _do_command($command, $params, $type, $login=true, $add_token=true) {
		do {
			if ($login && !$this->token) {
				$result = $this->_do_login();
				// ak prihlasenie nebolo uspesne, nema zmysel pokracovat
				if ($result->state != self::LOGIN_OK_STATE) {
					return $result;
				}
				// kedze sme neboli lognuty, token sa tam nepridal
				if ($add_token) {
					$params[0] = $this->token;
				}
			}
			$result = $this->_do_soap($type, $command, $params);
			if ($result->state == self::EXPIRED_STATE) {
				// vynulujem nastavenia, kedze su uz neplatne
				$this->token = null;
				$this->web_backend = null;
				$this->mail_backend = null;
				$this->dns_backend = null;
				$this->nic_backend = null;
				$this->sms_backend = null;
				$this->invoicing_backend = null;
			}
			// ak sa nema robit re-login, returnem
			if (!$this->relogin_on_expire) {
				return $result;
			}
		// ak session exspirovala, urobim re-login
		} while ($result->state == self::EXPIRED_STATE);
		return $result;
	}

	function set_metadata($result) {
		$this->permissions = $result->data->permissions;
		$prop = array_keys(get_object_vars($result->data->services));
		if (is_array($result->data->services->{$prop[0]})) {
			$this->services = $result->data->services->{$prop[0]};
		} else {
			$this->services = array();
			$this->services[0] = $result->data->services->{$prop[0]};
		}
		// TODO: urobit univerzalnejsie
		foreach ($this->services as $i) {
			if ($i->type == "W") {
				$this->web_backend = $i->backends->W->api_wsdl;
				$this->mail_backend = $i->backends->M->api_wsdl;
				$this->dns_backend = $i->backends->D->api_wsdl;
			} elseif ($i->type == "N") {
				$this->nic_backend = $i->backends->N->api_wsdl;
			} elseif ($i->type == "S") {
				$this->sms_backend = $i->backends->S->api_wsdl;
			} elseif ($i->type == "I") {
				$this->invoicing_backend = $i->backends->I->api_wsdl;
			}
		}
	}

	private function _do_login() {
		$result = $this->_do_command("do_login", array($this->login, $this->password), "DI", false);
		if ($result->state == self::LOGIN_OK_STATE) {
			$this->token = $result->data->token;
			$this->set_metadata($result);
		}
		return $result;
	}

	function do_logout() {
		if ($this->token) {
			$result = $this->_do_command("do_logout", array($this->token), "DI", false);
			if ($result->state == "oksu") {
				$this->set_metadata($result);
			} else {
				$this->token = null;
				$this->web_backend = null;
				$this->mail_backend = null;
				$this->dns_backend = null;
				$this->nic_backend = null;
				$this->sms_backend = null;
				$this->invoicing_backend = null;
				$res = new stdClass();
				$res->state = "ok";
				return $res;
			}
		}
		return false;
	}

	function set_login($login, $password) {
		$this->login = $login;
		$this->password = $password;
	}

	function change_password($pass_old, $pass_new) {
		$result = $this->_do_command("change_password", array($this->token, $pass_old, $pass_new), "DI");
		// ak zmena hesla bola uspesna, aktualizujeme heslo aj v pamati
		if ($result->state == self::CHPASS_OK_STATE) {
			$this->password = $pass_new;
		}
		return $result;
	}

	function get_user_info() {
		return $this->_do_command("get_user_info", array($this->token), "DI");
	}

	function send_message($message, $service_type) {
		return $this->_do_command("send_message", array($this->token, $message, $service_type), "DI");
	}

	function get_cart_items() {
		return $this->_do_command("get_cart_items", array($this->token), "DI");
	}

	function remove_cart_item($itm_id) {
		return $this->_do_command("remove_cart_item", array($this->token, $itm_id), "DI");
	}

	function create_order_from_cart() {
		return $this->_do_command("create_order_from_cart", array($this->token), "DI");
	}

	function order_credit($count) {
		return $this->_do_command("order_credit", array($this->token, $count), "DI");
	}

	function get_credit() {
		return $this->_do_command("get_credit", array($this->token), "DI");
	}

	function get_credit_movements($poffset, $pcount, $fservice_type=NULL, $flabel=NULL, $fdateaddfrom=NULL, $fdateaddto=NULL) {
		return $this->_do_command("get_credit_movements", array($this->token, $poffset, $pcount, $fservice_type, $flabel, $fdateaddfrom, $fdateaddto), "DI");
	}

////////////////////////////////////////////////////////////////////////////////

	function check_token($token, $perms=null, $backend_name=null) {
		return $this->_do_command("check_token", array($token, $perms, $backend_name), "DI", false);
	}

	function document_change_notification($backend, $password, $documents) {
		return $this->_do_command("document_change_notification", array($backend, $password, $documents), "DI", false);
	}

	function create_notification($backend, $password, $notifications) {
		return $this->_do_command("create_notification", array($backend, $password, $notifications), "DI", false);
	}

////////////////////////////////////////////////////////////////////////////////

	function __destruct() {
		$this->do_logout();
	}
}
?>
